package org.drools.reteoo;

import java.util.ArrayList;
import java.util.List;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import org.junit.Before;

import org.junit.Test;
import org.kie.api.KieServices;
import org.kie.api.event.rule.AfterMatchFiredEvent;
import org.kie.api.event.rule.DefaultAgendaEventListener;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.FactHandle;

/**
 *
 * Activation of rule creates Activation fact which can be used to control other rules.
 */
public class DeclarativeAgendaTest {

    private KieSession ksessionBlock, ksessionUnblock;
    
    @Before
    public void prepareKieSessions() throws Exception {
        KieServices ks = KieServices.Factory.get();
        ksessionBlock = ks.getKieClasspathContainer().newKieSession("agendaBlockKSession");
        ksessionUnblock = ks.getKieClasspathContainer().newKieSession("agendaUnblockKSession");
    }

    @After
    public void disposeSessions() {
        if (ksessionBlock != null) {
            ksessionBlock.dispose();
        }
        if (ksessionUnblock != null) {
            ksessionUnblock.dispose();
        }
    }

    // test activation block together with agenda group
    @Test(timeout = 60000L)
    public void testActivationBlock() {
        OrderListener listener = new OrderListener();
        ksessionBlock.addEventListener(listener);

        // first run
        ksessionBlock.insert("startAgenda");
        ksessionBlock.insert("fireRules");
        FactHandle fireBlockerRule = ksessionBlock.insert("fireBlockerRule");
        ksessionBlock.fireAllRules();
        assertEquals(6, listener.size());  // this assertion fails, only 5 matches have fired with RETE
        String[] expected = { "startAgenda", "catering1", "sales1", "salesBlocker", "catering2", "salesBlocker" };
        for (int i = 0; i < listener.size(); i++) {
            assertEquals(expected[i], listener.get(i));
        }

        // second run
        ksessionBlock.delete(fireBlockerRule);
        ksessionBlock.removeEventListener(listener);
        listener = new OrderListener();
        ksessionBlock.addEventListener(listener);
        ksessionBlock.fireAllRules();

        assertEquals(1, listener.size());

        String[] expected2 = { "sales2" };
        for (int i = 0; i < listener.size(); i++) {
            assertEquals(expected2[i], listener.get(i));
        }
    }

    // testing unblockall command
    @Test(timeout = 60000L)
    public void testUnblockAll() {
        OrderListener listener = new OrderListener();
        ksessionUnblock.addEventListener(listener);

        // first run
        ksessionUnblock.insert("fireRules");
        ksessionUnblock.insert("fireBlockerRule");
        ksessionUnblock.fireAllRules();
        assertEquals(2, listener.size());
        String[] expected = { "salesBlocker", "salesBlocker" };
        for (int i = 0; i < listener.size(); i++) {
            assertEquals(expected[i], listener.get(i));
        }

        // second run
        ksessionUnblock.insert("fireUnblockerRule");
        ksessionUnblock.fireAllRules();
        assertEquals(8, listener.size());
        String[] expected2 = { "salesBlocker", "salesBlocker", "salesUnblocker", "sales1", "salesBlocker",
                "salesUnblocker", "sales2", "salesBlocker" };
        // the order of fired rules differs here for PHREAK and RETE - I assume this is intended, could you please confirm?
        for (int i = 0; i < listener.size(); i++) {
            assertEquals(expected2[i], listener.get(i));
        }
    }

    /**
     * Listener tracking the names of rules which fired while maintaining their order.
     */
    public static class OrderListener extends DefaultAgendaEventListener {

        private final List<String> rulesFired = new ArrayList<String>();

        @Override
        public void afterMatchFired(AfterMatchFiredEvent event) {
            rulesFired.add(event.getMatch().getRule().getName());
        }

        public int size() {
            return rulesFired.size();
        }

        public String get(int index) {
            return rulesFired.get(index);
        }
    }

}
